#requires -version 2.0


# Jeffery Hicks
# http://jdhitsolutions.com/blog
# follow on Twitter: http://twitter.com/JeffHicks
#
# 
# "Those who forget to script are doomed to repeat their work."

#  ****************************************************************
#  * DO NOT USE IN A PRODUCTION ENVIRONMENT UNTIL YOU HAVE TESTED *
#  * THOROUGHLY IN A LAB ENVIRONMENT. USE AT YOUR OWN RISK.  IF   *
#  * YOU DO NOT UNDERSTAND WHAT THIS SCRIPT DOES OR HOW IT WORKS, *
#  * DO NOT USE IT OUTSIDE OF A SECURE, TEST SETTING.             *
#  ****************************************************************


Function Get-Woeid {

<#
.Synopsis
   Retrieve WOEID information
.Description
    Search Yahoo's Geo Places for a WOEID (Where On Earth ID). You can search
    based on a postal code, or place name. Optionally, you can save the XML results
    to a file in your %TEMP% directory. The filename will start with the WOED.
.Parameter Search
    A search parameter such as a postal or zip code. You can also use a place name.
.Parameter SaveXML
    Save the XML results to a file in your %TEMP% directory.
.Example
PS C:\> get-woeid 89101

Country : US
Region  : Nevada
Postal  : 89101
Locale  : Las Vegas
WOEID   : 12795439

.Example
PS C:\> get-woeid "Helsinki, Finland"

Country : FI
Region  : Uudenmaan maakunta
Postal  :
Locale  : Helsinki
WOEID   : 565346
.Example
PS C:\> get-woeid "Helsinki, Finland" | get-weather


Date              : Mon, 26 Jul 2010 2:20 pm EEST
Location          : Helsinki, Finland
Temperature       : 84 F
Condition         : Fair
ForecastCondition : Partly Cloudy
ForecastLow       : 71 F
ForecastHigh      : 82 F
.Inputs
    strings
.Outputs
    custom object
.Link
http://jdhitsolutions.com/blog

    
.Notes
 NAME:      Get-Woeid
 VERSION:   1.0
 AUTHOR:    Jeffery Hicks
 LASTEDIT:  July 26, 2010
 
 Learn more with a copy of Windows PowerShell 2.0: TFM (SAPIEN Press 2010)
#>

Param(
[parameter(Position=0,Mandatory=$True,ValueFromPipeline=$True,
HelpMessage="Enter a place name or postal code.")]
[string[]]$search,
[switch]$SaveXML
)

Begin {
    Write-Verbose "Starting $($myinvocation.mycommand)"
    Write-Verbose "Creating Webclient"
    #Create .NET Webclient object
    $webclient=New-Object "System.Net.WebClient" 
}

Process {
    foreach ($item in $search) {
        Write-Verbose "Querying for $search"
        $url="https://query.yahooapis.com/v1/public/yql?q=select%20*%20from%20geo.places%20where%20text%3D'$ITEM'%20limit%201"
        Write-Verbose $url
        [xml]$xml=$webclient.DownloadString($url)
        if ($xml.query.results.place.woeid) {
            Write-Verbose "Parsing XML"
            $woeid=$xml.query.results.place.woeid
            $postal=$xml.query.results.place.postal.'#text'
            $country=$xml.query.results.place.country.code
            $region=$xml.query.results.place.admin1.'#text'
            $locale=$xml.query.results.place.locality1.'#text'
            Write-Verbose "Writing custom object"
            New-Object -TypeName PSObject -Property @{
                WOEID=$woeid
                Locale=$locale
                Region=$region
                Postal=$postal
                Country=$country
            }
            if ($SaveXML) {
              $file="$env:temp\$woeid.xml"
              Write-Verbose "Saving XML to $file"
              $xml.save($file)
            }
        } #if $xml
        else {
            Write-Warning "Failed to find anything for $item"
        }
    } #foreach
} #process

End {
    Write-Verbose "Ending $($myinvocation.mycommand)"
 }
} #end function



# ==============================================================================================
# 
# Microsoft PowerShell Source File -- Created with SAPIEN Technologies PrimalScript 2009
# 
# NAME:  Get-Weather
# 
# AUTHOR: Jeffery Hicks , JDH Information Technology Solutions
# DATE  : 7/25/2010
# 
# COMMENT: 
# 
# ==============================================================================================

Function Get-Weather {

<#
.Synopsis
   Retrieve weather information from Yahoo.com.
.Description
    Using the Yahoo weather APIs, this function will retrieve weather information for the
    specified WOEID or "Where on Earth ID". If you don't know your WOED, open a web
    browser and go to http://weather.yahoo.com/. Enter in your zip or postal code. Your
    WOEID will be the number at the end of the url. Or you can use the Get-Woeid function
    included in the JH-Weather module.
    
    A custom object is written to the pipeline and you can control how much information 
    is displayed by specifying -Basic, -Extended or -All. These switches control which 
    property set is displayed.
.Parameter Woeid
    Your WOEID, or "Where on Earth ID".
.Parameter Unit
	Indicate if you want to use Farenheit or Celsius formatting. The default is F.
.Parameter Basic
	Display properties Date, Location, Temperature, Condition, ForecastCondition,
	ForecastLow, and ForecastHigh. This is the default.
.Parameter Extended
	Display basic properties plus WindChill, WindSpeed, Humidity, Barometer,
	Visibility, and Tomorrow.
.Parameter All
	Display Extended properties plus Sunrise,Sunset,City,Region,Zip,Latitude,
	Longitude,URL
.Parameter Online
	Open the web page in your default browser.
.Example
    PS C:> get-weather 12795711

    Date              : Sun, 25 Jul 2010 11:51 am PDT
    Location          : Beverly Hills, CA 90210
	Temperature       : 68 F
	Condition         : Fair
	ForecastCondition : AM Clouds/PM Sun
	ForecastLow       : 61 F
	ForecastHigh      : 76 F
.Example
    PS C:\> get-weather -woeid 12795446 -extended

    Date              : Sun, 25 Jul 2010 11:53 am PDT
	Location          : Las Vegas, NV
	Temperature       : 101 F
	Condition         : Fair
	ForecastCondition : Sunny
	ForecastLow       : 89 F
	ForecastHigh      : 109 F
	WindChill         : 101 F
	WindSpeed         : 0 mph
	Humidity          : 16%
	Barometer         : 29.83 in and falling
	Visibility        : 10  mi
	Tomorrow          : Mon 26 Jul 2010 Partly Cloudy Low 86F: High: 105F
.Example
PS C:\> 12795711,12795446,12762736 | get-Weather -all | Select City,Temperature,Condition,Sunrise,Sunset

	City        : Beverly Hills
	Temperature : 68 F
	Condition   : Fair
	Sunrise     : 6:00 am
	Sunset      : 8:00 pm
	
	City        : Las Vegas
	Temperature : 101 F
	Condition   : Fair
	Sunrise     : 5:42 am
	Sunset      : 7:52 pm
	
	City        : Syracuse
	Temperature : 77 F
	Condition   : Mostly Cloudy
	Sunrise     : 5:47 am
	Sunset      : 8:33 pm
.Inputs
    Strings
.Outputs
    Custom object
.Link
http://jdhitsolutions.com/blog

.Link
Get-Woeid
Select-XML
    
.Notes
 NAME:      Get-Weather
 VERSION:   2.1
 AUTHOR:    Jeffery Hicks
 LASTEDIT:  July 25, 2010

 Learn more with a copy of Windows PowerShell 2.0: TFM (SAPIEN Press 2010)
#>


[cmdletbinding()]

Param(
    [Parameter(Position=0,Mandatory=$True,ValueFromPipeline=$True,
    ValueFromPipelineByPropertyName=$True,
    HelpMessage="Enter a WOEID value")]
    [string[]]$woeid,
    
    [Parameter(Position=1,ValueFromPipeline=$False)]
    [ValidateSet("f","c")]
    [string]$Unit="f",
    [switch]$Online,
    [switch]$Basic,
    [switch]$Extended,
    [switch]$All
    
   )

Begin {    
	Write-Verbose "Starting $($myinvocation.mycommand)"
	
	#define property sets
	$BasicProp=@("Date","Location","Temperature","Condition","ForecastCondition",
	"ForecastLow","ForecastHigh")
	$ExtendedProp=$BasicProp
	$ExtendedProp+=@("WindChill","WindSpeed","Humidity","Barometer",
	"Visibility","Tomorrow")
	$AllProp=$ExtendedProp
	$AllProp+=@("Sunrise","Sunset","City","Region","Latitude",
	"Longitude","ID","URL")
	
	#unit must be lower case
	$Unit=$Unit.ToLower()
	Write-Verbose "Using unit $($unit.toLower())"
    Write-Verbose "Creating Webclient"
    #Create .NET Webclient object
    $webclient=New-Object "System.Net.WebClient"
 }
 
Process {

	foreach ($id in $woeid) {
		Write-Verbose "Getting weather info for $id"
	    [string]$urlbase="http://weather.yahooapis.com/forecastrss"
	    [string]$url=$urlbase + "?w="+$id+"&u="+$Unit
	    Write-Verbose "Downloading $url"
	    [xml]$xml=$webclient.DownloadString($url)
	    
	    #initialize a new hash table
	    $properties=@{}
	    if ($xml.rss.channel.item.Title -eq "City not found") {
	        Write-Warning "Could not find a location for $id"}
	    else {
	    	Write-Verbose "Processing xml"
	    	
	    	#get the yweather nodes	
        	#yweather information comes from a different namespace ao we'll use Select-XML
        	#to extract the data
        	#define the namespace hash
        	$namespace=@{yweather=$xml.rss.yweather}
        	$units=(Select-Xml -xml $xml -XPath "//yweather:units" -Namespace $namespace ).node
	    	
	        $properties.Add("Condition",$xml.rss.channel.item.condition.text)
	        $properties.Add("Temperature","$($xml.rss.channel.item.condition.temp) $($units.temperature)")
	        $properties.Add("Date",$xml.rss.channel.item.condition.date )
	    
	        #get forecast
	        $properties.add("ForecastDate",$xml.rss.channel.item.forecast[0].date)
	        $properties.add("ForecastCondition",$xml.rss.channel.item.forecast[0].text )
	        $properties.Add("ForecastLow","$($xml.rss.channel.item.forecast[0].low) $($units.temperature)" )
	        $properties.Add("ForecastHigh","$($xml.rss.channel.item.forecast[0].high) $($units.temperature)" )
	        
	        #build tomorrow's foreacst
	        $t=$xml.rss.channel.item.forecast[1]
	        $tomorrow="{0} {1} {2} Low {3}{4}: High: {5}{6}" -f $t.day,$t.Date,$t.Text,$t.low, $($units.temperature),$t.high, $($units.temperature)
	        $properties.add("Tomorrow",$tomorrow)
	        
	        #get optional information
          	$properties.Add("Latitude",$xml.rss.channel.item.lat)
        	$properties.Add("Longitude",$xml.rss.channel.item.long)
        	$city=$xml.rss.channel.location.city
        	$properties.Add("City",$city)
        	$region=$xml.rss.channel.location.region
            $country=$xml.rss.channel.location.country
        	
        	if (-not ($region)) {
               #if no region found then use country
               Write-Verbose "No region found. Using Country"
                $region=$country
            }
            $properties.Add("Region",$region)
            $location="{0}, {1}" -f $city,$region
	        $properties.Add("Location",$location)	    	 
	        $properties.Add("ID",$id)
        	#get additional yweather information        	
        	$wind=(select-xml -xml $xml -XPath "//yweather:wind" -Namespace $namespace ).node
        	$astronomy=(select-xml -xml $xml -XPath "//yweather:astronomy" -Namespace $namespace ).node
        	$atmosphere=(select-xml -xml $xml -XPath "//yweather:atmosphere" -Namespace $namespace ).node
        	
        	$properties.Add("WindChill","$($wind.chill) $($units.temperature)")
        	$properties.Add("WindSpeed","$($wind.speed) $($units.speed)")
        	$properties.Add("Humidity","$($atmosphere.humidity)%")
        	$properties.Add("Visibility","$($atmosphere.visibility)  $($units.distance)")
        	#decode rising
        	switch ($atmosphere.rising) {
        		0 {$state="steady"}
        		1 {$state="rising"}
        		2 {$state="falling"}
        	}
         	$properties.Add("Barometer","$($atmosphere.pressure) $($units.pressure) and $state")
        	$properties.Add("Sunrise",$astronomy.sunrise)
        	$properties.Add("Sunset",$astronomy.sunset)
        	$properties.Add("url",$url)
		    #create new object to hold values
		   $obj= New-Object -TypeName PSObject -Property $properties
		   
		   #write object and properties. Default is Basic
		   if ($All) {
		   	 Write-Verbose "Using All property set"
		   	 $obj | Select-Object -Property $AllProp
		    }
		   elseif ($Extended) {
		   	 Write-Verbose "Using Extended property set"
		   	 $obj | Select-Object -Property $ExtendedProp
		   }
		   else {
		   	 Write-Verbose "Using Basic property set"
		   	 $obj | Select-Object -Property $BasicProp
		   }
	    } #else
	    if ($online) {
	    	Write-Verbose "Opening $url in web browser"
	    	Start-Process -FilePath $url
	    }
   } #foreach $id
 } #process
 
 End {
 	Write-Verbose "Ending $($myinvocation.mycommand)"
 }   
   
} #end function

###############################################################################
Set-Alias -Name gw -Value get-weather
Set-Alias -Name gwid -Value get-woeid

Export-ModuleMember -alias * -Function *




